@@ -2,7 +2,7 @@
 /*
  +-------------------------------------------------------------------------+
  | Roundcube Webmail IMAP Client                                           |
- | Version 1.1.2                                                           |
+ | Version 1.1.4                                                           |
  |                                                                         |
  | Copyright (C) 2005-2015, The Roundcube Dev Team                         |
  |                                                                         |
@@ -131,7 +131,7 @@ if ($RCMAIL->task == 'login' && $RCMAIL->action == 'login') {
 
             // prevent redirect to compose with specified ID (#1488226)
             if ($query['_action'] == 'compose' && !empty($query['_id'])) {
-                $query = array();
+                $query = array('_action' => 'compose');
             }
         }
 
