@@ -21,7 +21,7 @@
 */
 
 // application constants
-define('RCMAIL_VERSION', '1.1.0');
+define('RCMAIL_VERSION', '1.1.4');
 define('RCMAIL_START', microtime(true));
 
 if (!defined('INSTALL_PATH')) {
@@ -68,11 +68,14 @@ spl_autoload_register('rcmail_autoload');
 // backward compatybility (to be removed)
 require_once INSTALL_PATH . 'program/include/bc.php';
 
-// load the UTF-8 portablity layer from Patchwork
-if (!function_exists('iconv') || !function_exists('utf8_encode') || !extension_loaded('mbstring')) {
-    \Patchwork\Utf8\Bootup::initAll();
+// load the UTF-8 portability layers from Patchwork
+// don't load mbstring layer as it conflicts with Roundcube Framework (#1490280)
+if (!function_exists('iconv')) {
+    \Patchwork\Utf8\Bootup::initIconv();
+}
+if (!function_exists('utf8_encode')) {
+    \Patchwork\Utf8\Bootup::initUtf8Encode();
 }
-
 
 /**
  * PHP5 autoloader routine for dynamic class loading
