@@ -35,50 +35,20 @@ class rcube_plugin_api
     public $url = 'plugins/';
     public $task = '';
     public $output;
-    public $handlers = array();
-    public $allowed_prefs = array();
+    public $handlers              = array();
+    public $allowed_prefs         = array();
+    public $allowed_session_prefs = array();
+    public $active_plugins        = array();
 
-    protected $plugins = array();
-    protected $tasks = array();
-    protected $actions = array();
-    protected $actionmap = array();
-    protected $objectsmap = array();
+    protected $plugins           = array();
+    protected $tasks             = array();
+    protected $actions           = array();
+    protected $actionmap         = array();
+    protected $objectsmap        = array();
     protected $template_contents = array();
-    protected $active_hook = false;
+    protected $exec_stack        = array();
+    protected $deprecated_hooks  = array();
 
-    // Deprecated names of hooks, will be removed after 0.5-stable release
-    protected $deprecated_hooks = array(
-        'create_user'       => 'user_create',
-        'kill_session'      => 'session_destroy',
-        'upload_attachment' => 'attachment_upload',
-        'save_attachment'   => 'attachment_save',
-        'get_attachment'    => 'attachment_get',
-        'cleanup_attachments' => 'attachments_cleanup',
-        'display_attachment' => 'attachment_display',
-        'remove_attachment' => 'attachment_delete',
-        'outgoing_message_headers' => 'message_outgoing_headers',
-        'outgoing_message_body' => 'message_outgoing_body',
-        'address_sources'   => 'addressbooks_list',
-        'get_address_book'  => 'addressbook_get',
-        'create_contact'    => 'contact_create',
-        'save_contact'      => 'contact_update',
-        'contact_save'      => 'contact_update',
-        'delete_contact'    => 'contact_delete',
-        'manage_folders'    => 'folders_list',
-        'list_mailboxes'    => 'mailboxes_list',
-        'save_preferences'  => 'preferences_save',
-        'user_preferences'  => 'preferences_list',
-        'list_prefs_sections' => 'preferences_sections_list',
-        'list_identities'   => 'identities_list',
-        'create_identity'   => 'identity_create',
-        'delete_identity'   => 'identity_delete',
-        'save_identity'     => 'identity_update',
-        'identity_save'     => 'identity_update',
-        // to be removed after 0.8
-        'imap_init'         => 'storage_init',
-        'mailboxes_list'    => 'storage_folders',
-        'imap_connect'      => 'storage_connect',
-    );
 
     /**
      * This implements the 'singleton' design pattern
@@ -167,10 +137,12 @@ class rcube_plugin_api
      * Load the specified plugin
      *
      * @param string Plugin name
+     * @param boolean Force loading of the plugin even if it doesn't match the filter
+     * @param boolean Require loading of the plugin, error if it doesn't exist
      *
      * @return boolean True on success, false if not loaded or failure
      */
-    public function load_plugin($plugin_name)
+    public function load_plugin($plugin_name, $force = false, $require = true)
     {
         static $plugins_dir;
 
@@ -180,23 +152,26 @@ class rcube_plugin_api
         }
 
         // plugin already loaded
-        if ($this->plugins[$plugin_name] || class_exists($plugin_name, false)) {
+        if ($this->plugins[$plugin_name]) {
             return true;
         }
 
-        $fn = $plugins_dir . DIRECTORY_SEPARATOR . $plugin_name
-            . DIRECTORY_SEPARATOR . $plugin_name . '.php';
+        $fn = "$plugins_dir/$plugin_name/$plugin_name.php";
 
-        if (file_exists($fn)) {
-            include $fn;
+        if (is_readable($fn)) {
+            if (!class_exists($plugin_name, false)) {
+                include $fn;
+            }
 
             // instantiate class if exists
             if (class_exists($plugin_name, false)) {
                 $plugin = new $plugin_name($this);
+                $this->active_plugins[] = $plugin_name;
+
                 // check inheritance...
                 if (is_subclass_of($plugin, 'rcube_plugin')) {
                     // ... task, request type and framed mode
-                    if ((!$plugin->task || preg_match('/^('.$plugin->task.')$/i', $this->task))
+                    if (($force || !$plugin->task || preg_match('/^('.$plugin->task.')$/i', $this->task))
                         && (!$plugin->noajax || (is_object($this->output) && $this->output->type == 'html'))
                         && (!$plugin->noframe || empty($_REQUEST['_framed']))
                     ) {
@@ -218,7 +193,7 @@ class rcube_plugin_api
                     true, false);
             }
         }
-        else {
+        else if ($require) {
             rcube::raise_error(array('code' => 520, 'type' => 'php',
                 'file' => __FILE__, 'line' => __LINE__,
                 'message' => "Failed to load plugin file $fn"), true, false);
@@ -227,6 +202,145 @@ class rcube_plugin_api
         return false;
     }
 
+    /**
+     * Get information about a specific plugin.
+     * This is either provided my a plugin's info() method or extracted from a package.xml or a composer.json file
+     *
+     * @param string Plugin name
+     * @return array Meta information about a plugin or False if plugin was not found
+     */
+    public function get_info($plugin_name)
+    {
+        static $composer_lock, $license_uris = array(
+            'Apache'       => 'http://www.apache.org/licenses/LICENSE-2.0.html',
+            'Apache-2'     => 'http://www.apache.org/licenses/LICENSE-2.0.html',
+            'Apache-1'     => 'http://www.apache.org/licenses/LICENSE-1.0',
+            'Apache-1.1'   => 'http://www.apache.org/licenses/LICENSE-1.1',
+            'GPL'          => 'http://www.gnu.org/licenses/gpl.html',
+            'GPLv2'        => 'http://www.gnu.org/licenses/gpl-2.0.html',
+            'GPL-2.0'      => 'http://www.gnu.org/licenses/gpl-2.0.html',
+            'GPLv3'        => 'http://www.gnu.org/licenses/gpl-3.0.html',
+            'GPLv3+'       => 'http://www.gnu.org/licenses/gpl-3.0.html',
+            'GPL-3.0'      => 'http://www.gnu.org/licenses/gpl-3.0.html',
+            'GPL-3.0+'     => 'http://www.gnu.org/licenses/gpl.html',
+            'GPL-2.0+'     => 'http://www.gnu.org/licenses/gpl.html',
+            'AGPLv3'       => 'http://www.gnu.org/licenses/agpl.html',
+            'AGPLv3+'      => 'http://www.gnu.org/licenses/agpl.html',
+            'AGPL-3.0'     => 'http://www.gnu.org/licenses/agpl.html',
+            'LGPL'         => 'http://www.gnu.org/licenses/lgpl.html',
+            'LGPLv2'       => 'http://www.gnu.org/licenses/lgpl-2.0.html',
+            'LGPLv2.1'     => 'http://www.gnu.org/licenses/lgpl-2.1.html',
+            'LGPLv3'       => 'http://www.gnu.org/licenses/lgpl.html',
+            'LGPL-2.0'     => 'http://www.gnu.org/licenses/lgpl-2.0.html',
+            'LGPL-2.1'     => 'http://www.gnu.org/licenses/lgpl-2.1.html',
+            'LGPL-3.0'     => 'http://www.gnu.org/licenses/lgpl.html',
+            'LGPL-3.0+'    => 'http://www.gnu.org/licenses/lgpl.html',
+            'BSD'          => 'http://opensource.org/licenses/bsd-license.html',
+            'BSD-2-Clause' => 'http://opensource.org/licenses/BSD-2-Clause',
+            'BSD-3-Clause' => 'http://opensource.org/licenses/BSD-3-Clause',
+            'FreeBSD'      => 'http://opensource.org/licenses/BSD-2-Clause',
+            'MIT'          => 'http://www.opensource.org/licenses/mit-license.php',
+            'PHP'          => 'http://opensource.org/licenses/PHP-3.0',
+            'PHP-3'        => 'http://www.php.net/license/3_01.txt',
+            'PHP-3.0'      => 'http://www.php.net/license/3_0.txt',
+            'PHP-3.01'     => 'http://www.php.net/license/3_01.txt',
+        );
+
+        $dir  = dir($this->dir);
+        $fn   = unslashify($dir->path) . "/$plugin_name/$plugin_name.php";
+        $info = false;
+
+        if (!class_exists($plugin_name, false)) {
+            if (is_readable($fn)) {
+                include($fn);
+            }
+            else {
+                return false;
+            }
+        }
+
+        if (class_exists($plugin_name)) {
+            $info = $plugin_name::info();
+        }
+
+        // fall back to composer.json file
+        if (!$info) {
+            $composer = INSTALL_PATH . "/plugins/$plugin_name/composer.json";
+            if (is_readable($composer) && ($json = @json_decode(file_get_contents($composer), true))) {
+                list($info['vendor'], $info['name']) = explode('/', $json['name']);
+                $info['version'] = $json['version'];
+                $info['license'] = $json['license'];
+                $info['uri']     = $json['homepage'];
+                $info['require'] = array_filter(array_keys((array)$json['require']), function($pname) {
+                    if (strpos($pname, '/') == false) {
+                        return false;
+                    }
+                    list($vendor, $name) = explode('/', $pname);
+                    return !($name == 'plugin-installer' || $vendor == 'pear-pear');
+                });
+            }
+
+            // read local composer.lock file (once)
+            if (!isset($composer_lock)) {
+                $composer_lock = @json_decode(@file_get_contents(INSTALL_PATH . "/composer.lock"), true);
+                if ($composer_lock['packages']) {
+                    foreach ($composer_lock['packages'] as $i => $package) {
+                        $composer_lock['installed'][$package['name']] = $package;
+                    }
+                }
+            }
+
+            // load additional information from local composer.lock file
+            if ($lock = $composer_lock['installed'][$json['name']]) {
+                $info['version'] = $lock['version'];
+                $info['uri']     = $lock['homepage'] ? $lock['homepage'] : $lock['source']['uri'];
+                $info['src_uri'] = $lock['dist']['uri'] ? $lock['dist']['uri'] : $lock['source']['uri'];
+            }
+        }
+
+        // fall back to package.xml file
+        if (!$info) {
+            $package = INSTALL_PATH . "/plugins/$plugin_name/package.xml";
+            if (is_readable($package) && ($file = file_get_contents($package))) {
+                $doc = new DOMDocument();
+                $doc->loadXML($file);
+                $xpath = new DOMXPath($doc);
+                $xpath->registerNamespace('rc', "http://pear.php.net/dtd/package-2.0");
+
+                // XPaths of plugin metadata elements
+                $metadata = array(
+                    'name'    => 'string(//rc:package/rc:name)',
+                    'version' => 'string(//rc:package/rc:version/rc:release)',
+                    'license' => 'string(//rc:package/rc:license)',
+                    'license_uri' => 'string(//rc:package/rc:license/@uri)',
+                    'src_uri' => 'string(//rc:package/rc:srcuri)',
+                    'uri'     => 'string(//rc:package/rc:uri)',
+                );
+
+                foreach ($metadata as $key => $path) {
+                    $info[$key] = $xpath->evaluate($path);
+                }
+
+                // dependent required plugins (can be used, but not included in config)
+                $deps = $xpath->evaluate('//rc:package/rc:dependencies/rc:required/rc:package/rc:name');
+                for ($i = 0; $i < $deps->length; $i++) {
+                    $dn = $deps->item($i)->nodeValue;
+                    $info['require'][] = $dn;
+                }
+            }
+        }
+
+        // At least provide the name
+        if (!$info && class_exists($plugin_name)) {
+            $info = array('name' => $plugin_name, 'version' => '--');
+        }
+        else if ($info['license'] && empty($info['license_uri']) && ($license_uri = $license_uris[$info['license']])) {
+            $info['license_uri'] = $license_uri;
+        }
+
+        return $info;
+    }
+
     /**
      * Allows a plugin object to register a callback for a certain hook
      *
@@ -260,9 +374,11 @@ class rcube_plugin_api
      */
     public function unregister_hook($hook, $callback)
     {
-        $callback_id = array_search($callback, $this->handlers[$hook]);
+        $callback_id = array_search($callback, (array) $this->handlers[$hook]);
         if ($callback_id !== false) {
-            unset($this->handlers[$hook][$callback_id]);
+            // array_splice() removes the element and re-indexes keys
+            // that is required by the 'for' loop in exec_hook() below
+            array_splice($this->handlers[$hook], $callback_id, 1);
         }
     }
 
@@ -281,21 +397,24 @@ class rcube_plugin_api
             $args = array('arg' => $args);
         }
 
-        $args += array('abort' => false);
-        $this->active_hook = $hook;
+        // TODO: avoid recursion by checking in_array($hook, $this->exec_stack) ?
 
-        foreach ((array)$this->handlers[$hook] as $callback) {
-            $ret = call_user_func($callback, $args);
+        $args += array('abort' => false);
+        array_push($this->exec_stack, $hook);
+
+        // Use for loop here, so handlers added in the hook will be executed too
+        for ($i = 0; $i < count($this->handlers[$hook]); $i++) {
+            $ret = call_user_func($this->handlers[$hook][$i], $args);
             if ($ret && is_array($ret)) {
                 $args = $ret + $args;
             }
 
-            if ($args['abort']) {
+            if ($args['break']) {
                 break;
             }
         }
 
-        $this->active_hook = false;
+        array_pop($this->exec_stack);
         return $args;
     }
 
@@ -378,7 +497,7 @@ class rcube_plugin_api
     /**
      * Register this plugin to be responsible for a specific task
      *
-     * @param string $task Task name (only characters [a-z0-9_.-] are allowed)
+     * @param string $task Task name (only characters [a-z0-9_-] are allowed)
      * @param string $owner Plugin name that registers this action
      */
     public function register_task($task, $owner)
@@ -388,7 +507,7 @@ class rcube_plugin_api
             return true;
         }
 
-        if ($task != asciiwords($task)) {
+        if ($task != asciiwords($task, true)) {
             rcube::raise_error(array('code' => 526, 'type' => 'php',
                 'file' => __FILE__, 'line' => __LINE__,
                 'message' => "Invalid task name: $task."
@@ -431,7 +550,7 @@ class rcube_plugin_api
      */
     public function is_processing($hook = null)
     {
-        return $this->active_hook && (!$hook || $this->active_hook == $hook);
+        return count($this->exec_stack) > 0 && (!$hook || in_array($hook, $this->exec_stack));
     }
 
     /**
@@ -482,6 +601,16 @@ class rcube_plugin_api
         return array_keys($this->plugins);
     }
 
+    /**
+     * Returns loaded plugin
+     *
+     * @return rcube_plugin Plugin instance
+     */
+    public function get_plugin($name)
+    {
+        return $this->plugins[$name];
+    }
+
     /**
      * Callback for template_container hooks
      *
